@@ -134,6 +134,7 @@ static void gst_gnome_vfs_src_get_property (GObject * object, guint prop_id,
 static gboolean gst_gnome_vfs_src_stop (GstBaseSrc * src);
 static gboolean gst_gnome_vfs_src_start (GstBaseSrc * src);
 static gboolean gst_gnome_vfs_src_is_seekable (GstBaseSrc * src);
+static gboolean gst_gnome_vfs_src_check_get_range (GstBaseSrc * src);
 static gboolean gst_gnome_vfs_src_get_size (GstBaseSrc * src, guint64 * size);
 static GstFlowReturn gst_gnome_vfs_src_create (GstBaseSrc * basesrc,
     guint64 offset, guint size, GstBuffer ** buffer);
@@ -254,6 +255,8 @@ gst_gnome_vfs_src_class_init (GstGnomeVFSSrcClass * klass)
   gstbasesrc_class->get_size = GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_get_size);
   gstbasesrc_class->is_seekable =
       GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_is_seekable);
+  gstbasesrc_class->check_get_range =
+      GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_check_get_range);
   gstbasesrc_class->create = GST_DEBUG_FUNCPTR (gst_gnome_vfs_src_create);
 }
 
@@ -525,32 +528,51 @@ audiocast_init (GstGnomeVFSSrc * src)
 
   if (!src->iradio_mode)
     return TRUE;
+
   GST_DEBUG_OBJECT (src, "audiocast: registering listener");
   if (audiocast_register_listener (&src->audiocast_port,
-          &src->audiocast_fd) < 0) {
-    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
-        ("Unable to listen on UDP port %d", src->audiocast_port));
-    close (src->audiocast_fd);
-    return FALSE;
-  }
+          &src->audiocast_fd) < 0)
+    goto no_listener;
+
   GST_DEBUG_OBJECT (src, "audiocast: creating pipe");
   src->audiocast_notify_queue = NULL;
-  if (pipe (pipefds) < 0) {
-    close (src->audiocast_fd);
-    return FALSE;
-  }
+  if (pipe (pipefds) < 0)
+    goto no_pipe;
+
   src->audiocast_thread_die_infd = pipefds[0];
   src->audiocast_thread_die_outfd = pipefds[1];
+
   GST_DEBUG_OBJECT (src, "audiocast: creating audiocast thread");
   src->audiocast_thread =
       g_thread_create ((GThreadFunc) audiocast_thread_run, src, TRUE, &error);
-  if (error != NULL) {
-    GST_ELEMENT_ERROR (src, RESOURCE, TOO_LAZY, (NULL),
-        ("Unable to create thread: %s", error->message));
+  if (error != NULL)
+    goto no_thread;
+
+  return TRUE;
+
+  /* ERRORS */
+no_listener:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
+        ("Unable to listen on UDP port %d", src->audiocast_port));
+    return FALSE;
+  }
+no_pipe:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
+        ("Unable to create socketpair"));
     close (src->audiocast_fd);
     return FALSE;
   }
-  return TRUE;
+no_thread:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, TOO_LAZY, (NULL),
+        ("Unable to create thread: %s", error->message));
+    close (src->audiocast_fd);
+    close (pipefds[0]);
+    close (pipefds[1]);
+    return FALSE;
+  }
 }
 
 static int
@@ -583,6 +605,8 @@ audiocast_register_listener (gint * port, gint * fd)
   *fd = sock;
 
   return 0;
+
+  /* ERRORS */
 lose_and_close:
   close (sock);
 lose:
@@ -969,12 +993,12 @@ gst_gnome_vfs_src_create (GstBaseSrc * basesrc, guint64 offset, guint size,
     res = gnome_vfs_read (src->handle, data,
         src->icy_metaint - src->icy_count, &readbytes);
 
+    if (res == GNOME_VFS_ERROR_EOF || (res == GNOME_VFS_OK && readbytes == 0))
+      goto eos;
+
     if (res != GNOME_VFS_OK)
       goto read_failed;
 
-    if (readbytes == 0)
-      goto eos;
-
     src->icy_count += readbytes;
     GST_BUFFER_OFFSET (buf) = src->curoffset;
     GST_BUFFER_SIZE (buf) = readbytes;
@@ -992,14 +1016,14 @@ gst_gnome_vfs_src_create (GstBaseSrc * basesrc, guint64 offset, guint size,
 
     res = gnome_vfs_read (src->handle, data, size, &readbytes);
 
+    if (res == GNOME_VFS_ERROR_EOF || (res == GNOME_VFS_OK && readbytes == 0))
+      goto eos;
+
     GST_BUFFER_SIZE (buf) = readbytes;
 
     if (res != GNOME_VFS_OK)
       goto read_failed;
 
-    if (readbytes == 0)
-      goto eos;
-
     src->curoffset += readbytes;
   }
 
@@ -1010,30 +1034,30 @@ gst_gnome_vfs_src_create (GstBaseSrc * basesrc, guint64 offset, guint size,
 
 seek_failed:
   {
-    GST_ERROR_OBJECT (src,
-        "Failed to seek to requested position %lld: %s",
-        offset, gnome_vfs_result_to_string (res));
+    GST_ELEMENT_ERROR (src, RESOURCE, SEEK, (NULL),
+        ("Failed to seek to requested position %" G_GINT64_FORMAT ": %s",
+            offset, gnome_vfs_result_to_string (res)));
     return GST_FLOW_ERROR;
   }
 cannot_seek:
   {
-    GST_ERROR_OBJECT (src,
-        "Requested seek from %lld to %lld on non-seekable stream",
-        src->curoffset, offset);
+    GST_ELEMENT_ERROR (src, RESOURCE, SEEK, (NULL),
+        ("Requested seek from %" G_GINT64_FORMAT " to %" G_GINT64_FORMAT
+            "on non-seekable stream", src->curoffset, offset));
     return GST_FLOW_ERROR;
   }
 read_failed:
   {
     gst_buffer_unref (buf);
-    GST_ERROR_OBJECT (src, "Failed to read data: %s",
-        gnome_vfs_result_to_string (res));
+    GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
+        ("Failed to read data: %s", gnome_vfs_result_to_string (res)));
     return GST_FLOW_ERROR;
   }
 eos:
   {
     gst_buffer_unref (buf);
-    GST_LOG_OBJECT (src, "Reading data gave EOS");
-    return GST_FLOW_WRONG_STATE;
+    GST_DEBUG_OBJECT (src, "Reading data gave EOS");
+    return GST_FLOW_UNEXPECTED;
   }
 }
 
@@ -1047,6 +1071,51 @@ gst_gnome_vfs_src_is_seekable (GstBaseSrc * basesrc)
   return src->seekable;
 }
 
+static gboolean
+gst_gnome_vfs_src_check_get_range (GstBaseSrc * basesrc)
+{
+  GstGnomeVFSSrc *src;
+  const gchar *protocol;
+
+  src = GST_GNOME_VFS_SRC (basesrc);
+
+  if (src->uri == NULL) {
+    GST_WARNING_OBJECT (src, "no URI set yet");
+    return FALSE;
+  }
+
+  if (gnome_vfs_uri_is_local (src->uri)) {
+    GST_LOG_OBJECT (src, "local URI (%s), assuming random access is possible",
+        GST_STR_NULL (src->uri_name));
+    return TRUE;
+  }
+
+  /* blacklist certain protocols we know won't work getrange-based */
+  protocol = gnome_vfs_uri_get_scheme (src->uri);
+  if (protocol == NULL)
+    goto undecided;
+
+  if (strcmp (protocol, "http") == 0) {
+    GST_LOG_OBJECT (src, "blacklisted protocol '%s', no random access possible"
+        " (URI=%s)", protocol, GST_STR_NULL (src->uri_name));
+    return FALSE;
+  }
+
+  /* fall through to undecided */
+
+undecided:
+  {
+    /* don't know what to do, let the basesrc class decide for us */
+    GST_LOG_OBJECT (src, "undecided about URI '%s', let base class handle it",
+        GST_STR_NULL (src->uri_name));
+
+    if (GST_BASE_SRC_CLASS (parent_class)->check_get_range)
+      return GST_BASE_SRC_CLASS (parent_class)->check_get_range (basesrc);
+
+    return FALSE;
+  }
+}
+
 static gboolean
 gst_gnome_vfs_src_get_size (GstBaseSrc * basesrc, guint64 * size)
 {
@@ -1064,7 +1133,7 @@ gst_gnome_vfs_src_get_size (GstBaseSrc * basesrc, guint64 * size)
   return TRUE;
 }
 
-/* open the file, do stuff necessary to go to READY state */
+/* open the file, do stuff necessary to go to PAUSED state */
 static gboolean
 gst_gnome_vfs_src_start (GstBaseSrc * basesrc)
 {
@@ -1080,30 +1149,13 @@ gst_gnome_vfs_src_start (GstBaseSrc * basesrc)
   gst_gnome_vfs_src_push_callbacks (src);
 
   if (src->uri != NULL) {
+    /* this can block... */
     if ((res = gnome_vfs_open_uri (&src->handle, src->uri,
-                GNOME_VFS_OPEN_READ)) != GNOME_VFS_OK) {
-      gchar *filename = gnome_vfs_uri_to_string (src->uri,
-          GNOME_VFS_URI_HIDE_PASSWORD);
-
-      gst_gnome_vfs_src_pop_callbacks (src);
-      audiocast_thread_kill (src);
-
-      if (res == GNOME_VFS_ERROR_NOT_FOUND) {
-        GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND, (NULL),
-            ("Could not open vfs file \"%s\" for reading: %s",
-                filename, gnome_vfs_result_to_string (res)));
-      } else {
-        GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
-            ("Could not open vfs file \"%s\" for reading: %s",
-                filename, gnome_vfs_result_to_string (res)));
-      }
-      g_free (filename);
-      return FALSE;
-    }
+                GNOME_VFS_OPEN_READ)) != GNOME_VFS_OK)
+      goto open_failed;
     src->own_handle = TRUE;
   } else if (!src->handle) {
-    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL), ("No filename given"));
-    return FALSE;
+    goto no_filename;
   } else {
     src->own_handle = FALSE;
   }
@@ -1133,6 +1185,34 @@ gst_gnome_vfs_src_start (GstBaseSrc * basesrc)
   }
 
   return TRUE;
+
+  /* ERRORS */
+open_failed:
+  {
+    gchar *filename = gnome_vfs_uri_to_string (src->uri,
+        GNOME_VFS_URI_HIDE_PASSWORD);
+
+    gst_gnome_vfs_src_pop_callbacks (src);
+    audiocast_thread_kill (src);
+
+    if (res == GNOME_VFS_ERROR_NOT_FOUND ||
+        res == GNOME_VFS_ERROR_SERVICE_NOT_AVAILABLE) {
+      GST_ELEMENT_ERROR (src, RESOURCE, NOT_FOUND, (NULL),
+          ("Could not open vfs file \"%s\" for reading: %s",
+              filename, gnome_vfs_result_to_string (res)));
+    } else {
+      GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
+          ("Could not open vfs file \"%s\" for reading: %s",
+              filename, gnome_vfs_result_to_string (res)));
+    }
+    g_free (filename);
+    return FALSE;
+  }
+no_filename:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL), ("No filename given"));
+    return FALSE;
+  }
 }
 
 static gboolean
