@@ -25,7 +25,7 @@
 #endif
 
 #include <gst/gst.h>
-#include <theora/theoradec.h>
+#include <theora/theora.h>
 #include <string.h>
 
 G_BEGIN_DECLS
@@ -35,10 +35,10 @@ G_BEGIN_DECLS
 #define GST_THEORA_DEC(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj),GST_TYPE_THEORA_DEC,GstTheoraDec))
 #define GST_THEORA_DEC_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_THEORA_DEC,GstTheoraDecClass))
+  (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_THEORA_DEC,GstTheoraDec))
 #define GST_IS_THEORA_DEC(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_THEORA_DEC))
-#define GST_IS_THEORA_DEC_CLASS(klass) \
+#define GST_IS_THEORA_DEC_CLASS(obj) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_THEORA_DEC))
 
 typedef struct _GstTheoraDec GstTheoraDec;
@@ -58,54 +58,30 @@ struct _GstTheoraDec
   GstPad *srcpad;
 
   /* theora decoder state */
-  th_dec_ctx *decoder;
-  //theora_state state;
-  th_setup_info *setup;
-  th_info info;
-  th_comment comment;
+  theora_state state;
+  theora_info info;
+  theora_comment comment;
 
   gboolean have_header;
+  guint64 granulepos;
+  guint64 granule_shift;
 
   GstClockTime last_timestamp;
-  guint64 frame_nr;
+  guint64 frame_nr; /* unused */
   gboolean need_keyframe;
   gint width, height;
   gint offset_x, offset_y;
-  gint output_bpp;
-
-  /* telemetry debuging options */
-  gint telemetry_mv;
-  gint telemetry_mbmode;
-  gint telemetry_qi;
-  gint telemetry_bits;
 
   gboolean crop;
 
-  /* list of buffers that need timestamps */
   GList *queued;
-  /* list of raw output buffers */
-  GList *output;
-  /* gather/decode queues for reverse playback */
-  GList *gather;
-  GList *decode;
-  GList *pendingevents;
-
-  GstTagList *tags;
 
   /* segment info */ /* with STREAM_LOCK */
   GstSegment segment;
-  gboolean discont;
-  guint32 seqnum;
 
   /* QoS stuff */ /* with LOCK*/
-  gdouble proportion;
+  gboolean proportion;
   GstClockTime earliest_time;
-  guint64 processed;
-  guint64 dropped;
-
-  gboolean have_par;
-  gint par_num;
-  gint par_den;
 };
 
 struct _GstTheoraDecClass
@@ -113,8 +89,6 @@ struct _GstTheoraDecClass
   GstElementClass parent_class;
 };
 
-GType gst_theora_dec_get_type (void);
-
 G_END_DECLS
 
 #endif /* __GST_THEORADEC_H__ */
