@@ -28,11 +28,29 @@
  * video codec maintained by the <ulink url="http://www.xiph.org/">Xiph.org
  * Foundation</ulink>, based on the VP3 codec.
  * </para>
+ * <para>
+ * The theora codec internally only supports encoding of images that are a
+ * multiple of 16 pixls in both X and Y direction. It is however perfectly possible 
+ * to encode images with other dimensions because an arbitrary rectangular cropping
+ * region can be set up. This element will automatically set up a correct cropping
+ * region if the dimensions are not multiples of 16 pixels. The "border" and "center" 
+ * properties control how this cropping region will be set up. 
+ * </para>
+ * <para>
+ * To control the quality of the encoding, the "bitrate" and "quality" properties can
+ * be used. These two properties are mutualy exclusive. Setting the bitrate property
+ * will produce a constant bitrate (CBR) stream while setting the quality property
+ * will produce a variable bitrate (VBR) stream.
+ * </para>
  * <title>Example pipeline</title>
  * <programlisting>
  * gst-launch -v videotestsrc num-buffers=1000 ! theoraenc ! oggmux ! filesink location=videotestsrc.ogg
  * </programlisting>
+ * This example pipeline will encode a test video source to theora muxed in an
+ * ogg container. Refer to the theoradec documentation to decode the create stream.
  * </refsect2>
+ *
+ * Last reviewed on 2006-03-01 (0.10.4)
  */
 
 #ifdef HAVE_CONFIG_H
@@ -69,19 +87,6 @@ gst_border_mode_get_type (void)
 #define ROUND_UP_4(x) (((x) + 3) & ~3)
 #define ROUND_UP_8(x) (((x) + 7) & ~7)
 
-#define THEORA_DEF_CENTER               TRUE
-#define THEORA_DEF_BORDER               BORDER_BLACK
-#define THEORA_DEF_BITRATE              0
-#define THEORA_DEF_QUALITY              16
-#define THEORA_DEF_QUICK                TRUE
-#define THEORA_DEF_KEYFRAME_AUTO        TRUE
-#define THEORA_DEF_KEYFRAME_FREQ        64
-#define THEORA_DEF_KEYFRAME_FREQ_FORCE  64
-#define THEORA_DEF_KEYFRAME_THRESHOLD   80
-#define THEORA_DEF_KEYFRAME_MINDISTANCE 8
-#define THEORA_DEF_NOISE_SENSITIVITY    1
-#define THEORA_DEF_SHARPNESS            0
-
 /* taken from theora/lib/toplevel.c */
 static int
 _ilog (unsigned int v)
@@ -95,6 +100,18 @@ _ilog (unsigned int v)
   return (ret);
 }
 
+#define THEORA_DEF_CENTER               TRUE
+#define THEORA_DEF_BORDER               BORDER_BLACK
+#define THEORA_DEF_BITRATE              0
+#define THEORA_DEF_QUALITY              16
+#define THEORA_DEF_QUICK                TRUE
+#define THEORA_DEF_KEYFRAME_AUTO        TRUE
+#define THEORA_DEF_KEYFRAME_FREQ        64
+#define THEORA_DEF_KEYFRAME_FREQ_FORCE  64
+#define THEORA_DEF_KEYFRAME_THRESHOLD   80
+#define THEORA_DEF_KEYFRAME_MINDISTANCE 8
+#define THEORA_DEF_NOISE_SENSITIVITY    1
+#define THEORA_DEF_SHARPNESS            0
 enum
 {
   ARG_0,
@@ -113,6 +130,25 @@ enum
   /* FILL ME */
 };
 
+/* this function does a straight granulepos -> timestamp conversion */
+static GstClockTime
+granulepos_to_timestamp (GstTheoraEnc * theoraenc, ogg_int64_t granulepos)
+{
+  guint64 iframe, pframe;
+  int shift = theoraenc->granule_shift;
+
+  if (granulepos < 0)
+    return GST_CLOCK_TIME_NONE;
+
+  iframe = granulepos >> shift;
+  pframe = granulepos - (iframe << shift);
+
+  /* num and den are 32 bit, so we can safely multiply with GST_SECOND */
+  return gst_util_uint64_scale ((guint64) (iframe + pframe),
+      GST_SECOND * theoraenc->info.fps_denominator,
+      theoraenc->info.fps_numerator);
+}
+
 static GstElementDetails theora_enc_details = {
   "TheoraEnc",
   "Codec/Encoder/Video",
@@ -148,7 +184,7 @@ static void theora_enc_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void theora_enc_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
-static void theora_enc_finalize (GObject *object);
+static void theora_enc_finalize (GObject * object);
 
 static void
 gst_theora_enc_base_init (gpointer g_class)
@@ -170,7 +206,7 @@ gst_theora_enc_class_init (GstTheoraEncClass * klass)
 
   gobject_class->set_property = theora_enc_set_property;
   gobject_class->get_property = theora_enc_get_property;
-  gobject_class->finalize     = theora_enc_finalize;
+  gobject_class->finalize = theora_enc_finalize;
 
   g_object_class_install_property (gobject_class, ARG_CENTER,
       g_param_spec_boolean ("center", "Center",
@@ -259,10 +295,10 @@ gst_theora_enc_init (GstTheoraEnc * enc, GstTheoraEncClass * g_class)
 }
 
 static void
-theora_enc_finalize (GObject *object)
+theora_enc_finalize (GObject * object)
 {
   GstTheoraEnc *enc = GST_THEORA_ENC (object);
-  
+
   GST_DEBUG_OBJECT (enc, "Finalizing");
   theora_clear (&enc->state);
   theora_comment_clear (&enc->comment);
@@ -348,7 +384,7 @@ theora_enc_sink_setcaps (GstPad * pad, GstCaps * caps)
 static guint64
 granulepos_add (guint64 granulepos, guint64 addend, gint shift)
 {
-  GstClockTime iframe, pframe;
+  guint64 iframe, pframe;
 
   iframe = granulepos >> shift;
   pframe = granulepos - (iframe << shift);
@@ -371,10 +407,14 @@ theora_buffer_from_packet (GstTheoraEnc * enc, ogg_packet * packet,
     goto no_buffer;
 
   memcpy (GST_BUFFER_DATA (buf), packet->packet, packet->bytes);
-  GST_BUFFER_OFFSET (buf) = enc->bytes_out;
+  /* see ext/ogg/README; OFFSET_END takes "our" granulepos, OFFSET its
+   * time representation */
   GST_BUFFER_OFFSET_END (buf) =
       granulepos_add (packet->granulepos, enc->granulepos_offset,
       enc->granule_shift);
+  GST_BUFFER_OFFSET (buf) = granulepos_to_timestamp (enc,
+      GST_BUFFER_OFFSET_END (buf));
+
   GST_BUFFER_TIMESTAMP (buf) = timestamp + enc->timestamp_offset;
   GST_BUFFER_DURATION (buf) = duration;
 
@@ -435,17 +475,17 @@ theora_set_header_on_caps (GstCaps * caps, GstBuffer * buf1,
   caps = gst_caps_make_writable (caps);
   structure = gst_caps_get_structure (caps, 0);
 
+  /* mark buffers */
+  GST_BUFFER_FLAG_SET (buf1, GST_BUFFER_FLAG_IN_CAPS);
+  GST_BUFFER_FLAG_SET (buf2, GST_BUFFER_FLAG_IN_CAPS);
+  GST_BUFFER_FLAG_SET (buf3, GST_BUFFER_FLAG_IN_CAPS);
+
   /* Copy buffers, because we can't use the originals -
    * it creates a circular refcount with the caps<->buffers */
   buf1 = gst_buffer_copy (buf1);
   buf2 = gst_buffer_copy (buf2);
   buf3 = gst_buffer_copy (buf3);
 
-  /* mark buffers */
-  GST_BUFFER_FLAG_SET (buf1, GST_BUFFER_FLAG_IN_CAPS);
-  GST_BUFFER_FLAG_SET (buf2, GST_BUFFER_FLAG_IN_CAPS);
-  GST_BUFFER_FLAG_SET (buf3, GST_BUFFER_FLAG_IN_CAPS);
-
   /* put copies of the buffers in a fixed list */
   g_value_init (&array, GST_TYPE_ARRAY);
 
