@@ -17,6 +17,30 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-vorbisdec
+ * @short_description: a decoder that decodes Vorbis to raw audio
+ * @see_also: vorbisenc, oggdemux
+ *
+ * <refsect2>
+ * <para>
+ * This element decodes a Vorbis stream to raw float audio.
+ * <ulink url="http://www.vorbis.com/">Vorbis</ulink> is a royalty-free
+ * audio codec maintained by the <ulink url="http://www.xiph.org/">Xiph.org
+ * Foundation</ulink>.
+ * </para>
+ * <title>Example pipelines</title>
+ * <para>
+ * <programlisting>
+ * gst-launch -v filesrc location=sine.ogg ! oggdemux ! vorbisdec ! audioconvert ! alsasink
+ * </programlisting>
+ * Decode an Ogg/Vorbis. To create an Ogg/Vorbis file refer to the documentation of vorbisenc.
+ * </para>
+ * </refsect2>
+ *
+ * Last reviewed on 2006-03-01 (0.10.4)
+ */
+
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
@@ -230,7 +254,9 @@ vorbis_dec_convert (GstPad * pad,
         case GST_FORMAT_BYTES:
           scale = sizeof (float) * dec->vi.channels;
         case GST_FORMAT_DEFAULT:
-          *dest_value = scale * (src_value * dec->vi.rate / GST_SECOND);
+          *dest_value =
+              scale * gst_util_uint64_scale_int (src_value, dec->vi.rate,
+              GST_SECOND);
           break;
         default:
           res = FALSE;
@@ -243,7 +269,7 @@ vorbis_dec_convert (GstPad * pad,
           break;
         case GST_FORMAT_TIME:
           *dest_value =
-              gst_util_uint64_scale (src_value, GST_SECOND, dec->vi.rate);
+              gst_util_uint64_scale_int (src_value, GST_SECOND, dec->vi.rate);
           break;
         default:
           res = FALSE;
@@ -255,7 +281,7 @@ vorbis_dec_convert (GstPad * pad,
           *dest_value = src_value / (sizeof (float) * dec->vi.channels);
           break;
         case GST_FORMAT_TIME:
-          *dest_value = gst_util_uint64_scale (src_value, GST_SECOND,
+          *dest_value = gst_util_uint64_scale_int (src_value, GST_SECOND,
               dec->vi.rate * sizeof (float) * dec->vi.channels);
           break;
         default:
@@ -329,7 +355,7 @@ vorbis_dec_src_query (GstPad * pad, GstQuery * query)
       break;
     }
     default:
-      res = FALSE;
+      res = gst_pad_query_default (pad, query);
       break;
   }
   return res;
@@ -363,7 +389,7 @@ vorbis_dec_sink_query (GstPad * pad, GstQuery * query)
       break;
     }
     default:
-      res = FALSE;
+      res = gst_pad_query_default (pad, query);
       break;
   }
 error:
@@ -651,20 +677,19 @@ vorbis_handle_header_packet (GstVorbisDec * vd, ogg_packet * packet)
   if (vorbis_synthesis_headerin (&vd->vi, &vd->vc, packet))
     goto header_read_error;
 
-  /* FIXME: we should probably double-check if packet[0] is 1/3/5 for each
-   * of these */
-  switch (packet->packetno) {
-    case 0:
+  switch (packet->packet[0]) {
+    case 0x01:
       res = vorbis_handle_identification_packet (vd);
       break;
-    case 1:
+    case 0x03:
       res = vorbis_handle_comment_packet (vd, packet);
       break;
-    case 2:
+    case 0x05:
       res = vorbis_handle_type_packet (vd);
       break;
     default:
       /* ignore */
+      g_warning ("unknown vorbis header packet found");
       res = GST_FLOW_OK;
       break;
   }
@@ -725,7 +750,7 @@ vorbis_dec_push (GstVorbisDec * dec, GstBuffer * buf)
 
         GST_BUFFER_OFFSET (buffer) = outoffset;
         GST_BUFFER_TIMESTAMP (buffer) =
-            gst_util_uint64_scale (outoffset, GST_SECOND, dec->vi.rate);
+            gst_util_uint64_scale_int (outoffset, GST_SECOND, dec->vi.rate);
         GST_DEBUG_OBJECT (dec, "patch buffer %" G_GUINT64_FORMAT
             " offset %" G_GUINT64_FORMAT, size, outoffset);
         size--;
@@ -793,10 +818,11 @@ vorbis_handle_data_packet (GstVorbisDec * vd, ogg_packet * packet)
   if (vd->granulepos != -1) {
     GST_BUFFER_OFFSET_END (out) = vd->granulepos + sample_count;
     GST_BUFFER_TIMESTAMP (out) =
-        gst_util_uint64_scale (vd->granulepos, GST_SECOND, vd->vi.rate);
+        gst_util_uint64_scale_int (vd->granulepos, GST_SECOND, vd->vi.rate);
   } else {
     GST_BUFFER_TIMESTAMP (out) = -1;
   }
+  /* this should not overflow */
   GST_BUFFER_DURATION (out) = sample_count * GST_SECOND / vd->vi.rate;
 
   if (vd->cur_timestamp != GST_CLOCK_TIME_NONE) {
