@@ -181,6 +181,10 @@ gst_base_audio_sink_provide_clock (GstElement * elem)
 
   sink = GST_BASE_AUDIO_SINK (elem);
 
+  /* we have no ringbuffer (must be NULL state */
+  if (sink->ringbuffer == NULL)
+    goto wrong_state;
+
   if (!gst_ring_buffer_is_acquired (sink->ringbuffer))
     goto wrong_state;
 
@@ -218,7 +222,8 @@ gst_base_audio_sink_get_time (GstClock * clock, GstBaseAudioSink * sink)
   /* our processed samples are always increasing */
   samples = gst_ring_buffer_samples_done (sink->ringbuffer);
 
-  result = samples * GST_SECOND / sink->ringbuffer->spec.rate;
+  result = gst_util_uint64_scale_int (samples, GST_SECOND,
+      sink->ringbuffer->spec.rate);
 
   return result;
 }
@@ -320,7 +325,7 @@ parse_error:
   {
     GST_DEBUG_OBJECT (sink, "could not parse caps");
     GST_ELEMENT_ERROR (sink, STREAM, FORMAT,
-        ("cannot parse audio format."), ("cannot parse audio format."));
+        (NULL), ("cannot parse audio format."));
     return FALSE;
   }
 acquire_error:
@@ -340,6 +345,9 @@ gst_base_audio_sink_get_times (GstBaseSink * bsink, GstBuffer * buffer,
   *end = GST_CLOCK_TIME_NONE;
 }
 
+/* FIXME, this waits for the drain to happen but it cannot be
+ * canceled.
+ */
 static gboolean
 gst_base_audio_sink_drain (GstBaseAudioSink * sink)
 {
@@ -364,6 +372,8 @@ gst_base_audio_sink_drain (GstBaseAudioSink * sink)
 
       GST_DEBUG_OBJECT (sink, "waiting for last sample to play");
       gst_clock_id_wait (id, NULL);
+
+      gst_clock_id_unref (id);
       sink->next_sample = -1;
     } else {
       GST_OBJECT_UNLOCK (sink);
@@ -414,8 +424,7 @@ gst_base_audio_sink_preroll (GstBaseSink * bsink, GstBuffer * buffer)
 wrong_state:
   {
     GST_DEBUG_OBJECT (sink, "ringbuffer in wrong state");
-    GST_ELEMENT_ERROR (sink, RESOURCE, NOT_FOUND,
-        ("sink not negotiated."), ("sink not negotiated."));
+    GST_ELEMENT_ERROR (sink, STREAM, FORMAT, (NULL), ("sink not negotiated."));
     return GST_FLOW_NOT_NEGOTIATED;
   }
 }
@@ -429,6 +438,9 @@ gst_base_audio_sink_get_offset (GstBaseAudioSink * sink)
 
   /* assume we can append to the previous sample */
   sample = sink->next_sample;
+  /* no previous sample, try to insert at position 0 */
+  if (sample == -1)
+    sample = 0;
 
   sps = sink->ringbuffer->samples_per_seg;
 
@@ -469,7 +481,7 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
 
   sink = GST_BASE_AUDIO_SINK (bsink);
 
-  if (GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT)) {
+  if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (buf, GST_BUFFER_FLAG_DISCONT))) {
     /* always resync after a discont */
     sink->next_sample = -1;
   }
@@ -477,13 +489,13 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   ringbuf = sink->ringbuffer;
 
   /* can't do anything when we don't have the device */
-  if (!gst_ring_buffer_is_acquired (ringbuf))
+  if (G_UNLIKELY (!gst_ring_buffer_is_acquired (ringbuf)))
     goto wrong_state;
 
   bps = ringbuf->spec.bytes_per_sample;
 
   size = GST_BUFFER_SIZE (buf);
-  if (size % bps != 0)
+  if (G_UNLIKELY (size % bps) != 0)
     goto wrong_size;
 
   samples = size / bps;
@@ -524,15 +536,17 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
   /* see if some clipping happened */
   diff = ctime - time;
   if (diff > 0) {
+    /* bring clipped time to samples */
     diff = gst_util_uint64_scale_int (diff, ringbuf->spec.rate, GST_SECOND);
     GST_DEBUG_OBJECT (sink, "clipping start to %" GST_TIME_FORMAT " %"
         G_GUINT64_FORMAT " samples", GST_TIME_ARGS (ctime), diff);
     samples -= diff;
-    data += samples * bps;
+    data += diff * bps;
     time = ctime;
   }
   diff = stop - cstop;
   if (diff > 0) {
+    /* bring clipped time to samples */
     diff = gst_util_uint64_scale_int (diff, ringbuf->spec.rate, GST_SECOND);
     GST_DEBUG_OBJECT (sink, "clipping stop to %" GST_TIME_FORMAT " %"
         G_GUINT64_FORMAT " samples", GST_TIME_ARGS (cstop), diff);
@@ -559,7 +573,7 @@ gst_base_audio_sink_render (GstBaseSink * bsink, GstBuffer * buf)
       GST_TIME_ARGS (render_time), render_offset, samples);
 
   /* roundoff errors in timestamp conversion */
-  if (sink->next_sample != -1) {
+  if (G_LIKELY (sink->next_sample != -1)) {
     diff = ABS ((gint64) render_offset - (gint64) sink->next_sample);
 
     /* we tollerate a 10th of a second diff before we start resyncing. This
@@ -622,6 +636,7 @@ no_sync:
 
   return GST_FLOW_OK;
 
+  /* SPECIAL cases */
 out_of_segment:
   {
     GST_DEBUG_OBJECT (sink,
@@ -630,19 +645,18 @@ out_of_segment:
         GST_TIME_ARGS (bsink->segment.start));
     return GST_FLOW_OK;
   }
+  /* ERRORS */
 wrong_state:
   {
     GST_DEBUG_OBJECT (sink, "ringbuffer not negotiated");
-    GST_ELEMENT_ERROR (sink, RESOURCE, NOT_FOUND,
-        ("sink not negotiated."), ("sink not negotiated."));
+    GST_ELEMENT_ERROR (sink, STREAM, FORMAT, (NULL), ("sink not negotiated."));
     return GST_FLOW_NOT_NEGOTIATED;
   }
 wrong_size:
   {
     GST_DEBUG_OBJECT (sink, "wrong size");
-    GST_ELEMENT_ERROR (sink, RESOURCE, NOT_FOUND,
-        ("sink received buffer of wrong size."),
-        ("sink received buffer of wrong size."));
+    GST_ELEMENT_ERROR (sink, STREAM, WRONG_TYPE,
+        (NULL), ("sink received buffer of wrong size."));
     return GST_FLOW_ERROR;
   }
 stopping:
@@ -690,29 +704,34 @@ gst_base_audio_sink_change_state (GstElement * element,
             gst_base_audio_sink_callback, sink);
       }
       if (!gst_ring_buffer_open_device (sink->ringbuffer))
-        return GST_STATE_CHANGE_FAILURE;
-      sink->next_sample = 0;
+        goto open_failed;
       break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       gst_ring_buffer_set_flushing (sink->ringbuffer, FALSE);
+      sink->next_sample = -1;
       break;
     case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
     {
       GstClock *clock;
+      GstClockTime time, base;
 
-      /* FIXME, only start slaving when we really start the ringbuffer */
       GST_OBJECT_LOCK (sink);
       clock = GST_ELEMENT_CLOCK (sink);
+      if (clock == NULL)
+        goto no_clock;
+
+      /* FIXME, only start slaving when we really start the ringbuffer */
       /* if we are slaved to a clock, we need to set the initial
        * calibration */
       if (clock != sink->provided_clock) {
-        GstClockTime time;
         GstClockTime rate_num, rate_denom;
 
+        base = element->base_time;
         time = gst_clock_get_internal_time (sink->provided_clock);
 
-        GST_DEBUG_OBJECT (sink, "time: %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (time));
+        GST_DEBUG_OBJECT (sink,
+            "time: %" GST_TIME_FORMAT " base: %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (time), GST_TIME_ARGS (base));
 
         gst_clock_set_master (sink->provided_clock, clock);
         /* FIXME, this is not yet accurate enough for smooth playback */
@@ -722,6 +741,7 @@ gst_base_audio_sink_change_state (GstElement * element,
         gst_clock_set_calibration (sink->provided_clock,
             time, element->base_time, rate_num, rate_denom);
       }
+    no_clock:
       GST_OBJECT_UNLOCK (sink);
       break;
     }
@@ -754,4 +774,10 @@ gst_base_audio_sink_change_state (GstElement * element,
   }
 
   return ret;
+
+open_failed:
+  {
+    GST_DEBUG_OBJECT (sink, "open failed");
+    return GST_STATE_CHANGE_FAILURE;
+  }
 }
