@@ -21,11 +21,23 @@
 /**
  * SECTION:gstringbuffer
  * @short_description: Base class for audio ringbuffer implementations
+ * @see_also: gstbaseaudiosink
  *
+ * <refsect2>
+ * <para>
  * This object is the base class for audio ringbuffers used by the base
  * audio source and sink classes.
+ * </para>
+ * <para>
+ * The ringbuffer abstracts a circular buffer of data. One reader and
+ * one writer can operate on the data from different threads in a lockfree 
+ * manner. The base class is sufficiently flexible to be used as an
+ * abstraction for DMA based ringbuffers as well as a pure software 
+ * implementations.
+ * </para>
+ * </refsect2>
  *
- * Last reviewed on 2005-11-24 (0.9.6)
+ * Last reviewed on 2006-02-02 (0.10.4)
  */
 
 #include <string.h>
@@ -50,7 +62,7 @@ gst_ring_buffer_get_type (void)
 {
   static GType ringbuffer_type = 0;
 
-  if (!ringbuffer_type) {
+  if (G_UNLIKELY (!ringbuffer_type)) {
     static const GTypeInfo ringbuffer_info = {
       sizeof (GstRingBufferClass),
       NULL,
@@ -394,7 +406,7 @@ gst_ring_buffer_open_device (GstRingBuffer * buf)
   GST_DEBUG_OBJECT (buf, "opening device");
 
   GST_OBJECT_LOCK (buf);
-  if (buf->open)
+  if (G_UNLIKELY (buf->open))
     goto was_opened;
 
   buf->open = TRUE;
@@ -403,10 +415,10 @@ gst_ring_buffer_open_device (GstRingBuffer * buf)
   g_assert (!buf->acquired);
 
   rclass = GST_RING_BUFFER_GET_CLASS (buf);
-  if (rclass->open_device)
+  if (G_LIKELY (rclass->open_device))
     res = rclass->open_device (buf);
 
-  if (!res)
+  if (G_UNLIKELY (!res))
     goto open_failed;
 
   GST_DEBUG_OBJECT (buf, "opened device");
@@ -454,19 +466,19 @@ gst_ring_buffer_close_device (GstRingBuffer * buf)
   GST_DEBUG_OBJECT (buf, "closing device");
 
   GST_OBJECT_LOCK (buf);
-  if (!buf->open)
+  if (G_UNLIKELY (!buf->open))
     goto was_closed;
 
-  if (buf->acquired)
+  if (G_UNLIKELY (buf->acquired))
     goto was_acquired;
 
   buf->open = FALSE;
 
   rclass = GST_RING_BUFFER_GET_CLASS (buf);
-  if (rclass->close_device)
+  if (G_LIKELY (rclass->close_device))
     res = rclass->close_device (buf);
 
-  if (!res)
+  if (G_UNLIKELY (!res))
     goto close_error;
 
   GST_DEBUG_OBJECT (buf, "closed device");
@@ -517,9 +529,7 @@ gst_ring_buffer_device_is_open (GstRingBuffer * buf)
   g_return_val_if_fail (GST_IS_RING_BUFFER (buf), FALSE);
 
   GST_OBJECT_LOCK (buf);
-
   res = buf->open;
-
   GST_OBJECT_UNLOCK (buf);
 
   return res;
@@ -552,22 +562,22 @@ gst_ring_buffer_acquire (GstRingBuffer * buf, GstRingBufferSpec * spec)
   GST_DEBUG_OBJECT (buf, "acquiring device");
 
   GST_OBJECT_LOCK (buf);
-  if (!buf->open)
+  if (G_UNLIKELY (!buf->open))
     goto not_opened;
 
-  if (buf->acquired)
+  if (G_UNLIKELY (buf->acquired))
     goto was_acquired;
 
   buf->acquired = TRUE;
 
   rclass = GST_RING_BUFFER_GET_CLASS (buf);
-  if (rclass->acquire)
+  if (G_LIKELY (rclass->acquire))
     res = rclass->acquire (buf, spec);
 
-  if (!res)
+  if (G_UNLIKELY (!res))
     goto acquire_failed;
 
-  if ((bps = buf->spec.bytes_per_sample) == 0)
+  if (G_UNLIKELY ((bps = buf->spec.bytes_per_sample) == 0))
     goto invalid_bps;
 
   segsize = buf->spec.segsize;
@@ -641,7 +651,7 @@ gst_ring_buffer_release (GstRingBuffer * buf)
   gst_ring_buffer_stop (buf);
 
   GST_OBJECT_LOCK (buf);
-  if (!buf->acquired)
+  if (G_UNLIKELY (!buf->acquired))
     goto was_released;
 
   buf->acquired = FALSE;
@@ -650,14 +660,14 @@ gst_ring_buffer_release (GstRingBuffer * buf)
   g_assert (buf->open == TRUE);
 
   rclass = GST_RING_BUFFER_GET_CLASS (buf);
-  if (rclass->release)
+  if (G_LIKELY (rclass->release))
     res = rclass->release (buf);
 
   /* signal any waiters */
   GST_DEBUG_OBJECT (buf, "signal waiter");
   GST_RING_BUFFER_SIGNAL (buf);
 
-  if (!res)
+  if (G_UNLIKELY (!res))
     goto release_failed;
 
   g_free (buf->empty_seg);
@@ -726,11 +736,9 @@ gst_ring_buffer_set_flushing (GstRingBuffer * buf, gboolean flushing)
   if (flushing) {
     gst_ring_buffer_pause_unlocked (buf);
   }
-
   GST_OBJECT_UNLOCK (buf);
 }
 
-
 /**
  * gst_ring_buffer_start:
  * @buf: the #GstRingBuffer to start
@@ -753,7 +761,7 @@ gst_ring_buffer_start (GstRingBuffer * buf)
   GST_DEBUG_OBJECT (buf, "starting ringbuffer");
 
   GST_OBJECT_LOCK (buf);
-  if (buf->abidata.ABI.flushing)
+  if (G_UNLIKELY (buf->abidata.ABI.flushing))
     goto flushing;
 
   /* if stopped, set to started */
@@ -776,14 +784,14 @@ gst_ring_buffer_start (GstRingBuffer * buf)
 
   rclass = GST_RING_BUFFER_GET_CLASS (buf);
   if (resume) {
-    if (rclass->resume)
+    if (G_LIKELY (rclass->resume))
       res = rclass->resume (buf);
   } else {
-    if (rclass->start)
+    if (G_LIKELY (rclass->start))
       res = rclass->start (buf);
   }
 
-  if (!res) {
+  if (G_UNLIKELY (!res)) {
     buf->state = GST_RING_BUFFER_STATE_PAUSED;
     GST_DEBUG_OBJECT (buf, "failed to start");
   } else {
@@ -814,26 +822,32 @@ gst_ring_buffer_pause_unlocked (GstRingBuffer * buf)
   res = g_atomic_int_compare_and_exchange (&buf->state,
       GST_RING_BUFFER_STATE_STARTED, GST_RING_BUFFER_STATE_PAUSED);
 
-  if (!res) {
-    /* was not started */
-    res = TRUE;
-    GST_DEBUG_OBJECT (buf, "was not started");
-    goto done;
-  }
+  if (!res)
+    goto not_started;
+
+  /* signal any waiters */
+  GST_DEBUG_OBJECT (buf, "signal waiter");
+  GST_RING_BUFFER_SIGNAL (buf);
 
   rclass = GST_RING_BUFFER_GET_CLASS (buf);
-  if (rclass->pause)
+  if (G_LIKELY (rclass->pause))
     res = rclass->pause (buf);
 
-  if (!res) {
+  if (G_UNLIKELY (!res)) {
     buf->state = GST_RING_BUFFER_STATE_STARTED;
     GST_DEBUG_OBJECT (buf, "failed to pause");
   } else {
     GST_DEBUG_OBJECT (buf, "paused");
   }
 
-done:
   return res;
+
+not_started:
+  {
+    /* was not started */
+    GST_DEBUG_OBJECT (buf, "was not started");
+    return TRUE;
+  }
 }
 
 /**
@@ -854,11 +868,10 @@ gst_ring_buffer_pause (GstRingBuffer * buf)
   g_return_val_if_fail (buf != NULL, FALSE);
 
   GST_OBJECT_LOCK (buf);
-  if (buf->abidata.ABI.flushing)
+  if (G_UNLIKELY (buf->abidata.ABI.flushing))
     goto flushing;
 
   res = gst_ring_buffer_pause_unlocked (buf);
-
   GST_OBJECT_UNLOCK (buf);
 
   return res;
@@ -908,10 +921,10 @@ gst_ring_buffer_stop (GstRingBuffer * buf)
   GST_RING_BUFFER_SIGNAL (buf);
 
   rclass = GST_RING_BUFFER_GET_CLASS (buf);
-  if (rclass->stop)
+  if (G_LIKELY (rclass->stop))
     res = rclass->stop (buf);
 
-  if (!res) {
+  if (G_UNLIKELY (!res)) {
     buf->state = GST_RING_BUFFER_STATE_STARTED;
     GST_DEBUG_OBJECT (buf, "failed to stop");
   } else {
@@ -940,17 +953,21 @@ guint
 gst_ring_buffer_delay (GstRingBuffer * buf)
 {
   GstRingBufferClass *rclass;
-  guint res = 0;
+  guint res;
 
   g_return_val_if_fail (buf != NULL, 0);
 
-  if (!gst_ring_buffer_is_acquired (buf))
-    return 0;
+  res = 0;
+
+  /* buffer must be acquired */
+  if (G_UNLIKELY (!gst_ring_buffer_is_acquired (buf)))
+    goto done;
 
   rclass = GST_RING_BUFFER_GET_CLASS (buf);
-  if (rclass->delay)
+  if (G_LIKELY (rclass->delay))
     res = rclass->delay (buf);
 
+done:
   return res;
 }
 
@@ -980,10 +997,9 @@ gst_ring_buffer_samples_done (GstRingBuffer * buf)
   /* and the number of samples not yet processed */
   delay = gst_ring_buffer_delay (buf);
 
-  samples = (segdone * buf->samples_per_seg);
-  raw = samples;
+  raw = samples = ((guint64) segdone) * buf->samples_per_seg;
 
-  if (samples >= delay)
+  if (G_LIKELY (samples >= delay))
     samples -= delay;
 
   GST_DEBUG_OBJECT (buf, "processed samples: raw %llu, delay %u, real %llu",
@@ -1014,7 +1030,7 @@ gst_ring_buffer_set_sample (GstRingBuffer * buf, guint64 sample)
   if (sample == -1)
     sample = 0;
 
-  if (buf->samples_per_seg == 0)
+  if (G_UNLIKELY (buf->samples_per_seg == 0))
     return;
 
   /* FIXME, we assume the ringbuffer can restart at a random 
@@ -1044,7 +1060,7 @@ gst_ring_buffer_clear_all (GstRingBuffer * buf)
   g_return_if_fail (buf != NULL);
 
   /* not fatal, we just are not negotiated yet */
-  if (buf->spec.segtotal <= 0)
+  if (G_UNLIKELY (buf->spec.segtotal <= 0))
     return;
 
   GST_DEBUG_OBJECT (buf, "clear all segments");
@@ -1059,26 +1075,30 @@ static gboolean
 wait_segment (GstRingBuffer * buf)
 {
   /* buffer must be started now or we deadlock since nobody is reading */
-  if (g_atomic_int_get (&buf->state) != GST_RING_BUFFER_STATE_STARTED) {
+  if (G_UNLIKELY (g_atomic_int_get (&buf->state) !=
+          GST_RING_BUFFER_STATE_STARTED)) {
     GST_DEBUG_OBJECT (buf, "start!");
     gst_ring_buffer_start (buf);
   }
 
   /* take lock first, then update our waiting flag */
   GST_OBJECT_LOCK (buf);
-  if (buf->abidata.ABI.flushing)
+  if (G_UNLIKELY (buf->abidata.ABI.flushing))
     goto flushing;
 
+  if (G_UNLIKELY (g_atomic_int_get (&buf->state) !=
+          GST_RING_BUFFER_STATE_STARTED))
+    goto not_started;
+
   if (g_atomic_int_compare_and_exchange (&buf->waiting, 0, 1)) {
     GST_DEBUG_OBJECT (buf, "waiting..");
-    if (g_atomic_int_get (&buf->state) != GST_RING_BUFFER_STATE_STARTED)
-      goto not_started;
-
     GST_RING_BUFFER_WAIT (buf);
-    if (buf->abidata.ABI.flushing)
+
+    if (G_UNLIKELY (buf->abidata.ABI.flushing))
       goto flushing;
 
-    if (g_atomic_int_get (&buf->state) != GST_RING_BUFFER_STATE_STARTED)
+    if (G_UNLIKELY (g_atomic_int_get (&buf->state) !=
+            GST_RING_BUFFER_STATE_STARTED))
       goto not_started;
   }
   GST_OBJECT_UNLOCK (buf);
@@ -1088,14 +1108,16 @@ wait_segment (GstRingBuffer * buf)
   /* ERROR */
 not_started:
   {
-    GST_OBJECT_UNLOCK (buf);
+    g_atomic_int_compare_and_exchange (&buf->waiting, 1, 0);
     GST_DEBUG_OBJECT (buf, "stopped processing");
+    GST_OBJECT_UNLOCK (buf);
     return FALSE;
   }
 flushing:
   {
-    GST_OBJECT_UNLOCK (buf);
+    g_atomic_int_compare_and_exchange (&buf->waiting, 1, 0);
     GST_DEBUG_OBJECT (buf, "flushing");
+    GST_OBJECT_UNLOCK (buf);
     return FALSE;
   }
 }
@@ -1111,7 +1133,7 @@ flushing:
  * @buf. The first sample should be written at position @sample in
  * the ringbuffer.
  *
- * @len not needs to be a multiple of the segment size of the ringbuffer
+ * @len does not need to be a multiple of the segment size of the ringbuffer
  * although it is recommended for optimal performance.
  *
  * Returns: The number of samples written to the ringbuffer or -1 on
@@ -1162,8 +1184,8 @@ gst_ring_buffer_commit (GstRingBuffer * buf, guint64 sample, guchar * data,
           ("pointer at %d, sample %llu, write to %d-%d, to_write %d, diff %d, segtotal %d, segsize %d",
           segdone, sample, writeseg, sampleoff, to_write, diff, segtotal, sps);
 
-      /* segment too far ahead, we need to drop */
-      if (diff < 0) {
+      /* segment too far ahead, we need to drop, hopefully UNLIKELY */
+      if (G_UNLIKELY (diff < 0)) {
         /* we need to drop one segment at a time, pretend we wrote a
          * segment. */
         sampleslen = MIN (sps, to_write);
@@ -1326,7 +1348,7 @@ not_started:
  * @len: the number of bytes to read
  *
  * Returns a pointer to memory where the data from segment @segment
- * can be found. This function is used by subclasses.
+ * can be found. This function is mostly used by subclasses.
  *
  * Returns: FALSE if the buffer is not started.
  *
@@ -1339,11 +1361,12 @@ gst_ring_buffer_prepare_read (GstRingBuffer * buf, gint * segment,
   guint8 *data;
   gint segdone;
 
+  g_return_val_if_fail (buf != NULL, FALSE);
+
   /* buffer must be started */
   if (g_atomic_int_get (&buf->state) != GST_RING_BUFFER_STATE_STARTED)
     return FALSE;
 
-  g_return_val_if_fail (buf != NULL, FALSE);
   g_return_val_if_fail (buf->data != NULL, FALSE);
   g_return_val_if_fail (segment != NULL, FALSE);
   g_return_val_if_fail (readptr != NULL, FALSE);
@@ -1358,7 +1381,8 @@ gst_ring_buffer_prepare_read (GstRingBuffer * buf, gint * segment,
   *len = buf->spec.segsize;
   *readptr = data + *segment * *len;
 
-  /* callback to fill the memory with data */
+  /* callback to fill the memory with data, for pull based
+   * scheduling. */
   if (buf->callback)
     buf->callback (buf, *readptr, *len, buf->cb_data);
 
@@ -1413,14 +1437,19 @@ gst_ring_buffer_clear (GstRingBuffer * buf, gint segment)
   guint8 *data;
 
   g_return_if_fail (buf != NULL);
-  /* no data means it's allready cleared */
-  if (buf->data == NULL)
+
+  /* no data means it's already cleared */
+  if (G_UNLIKELY (buf->data == NULL))
     return;
 
-  g_return_if_fail (buf->empty_seg != NULL);
+  /* no empty_seg means it's not opened */
+  if (G_UNLIKELY (buf->empty_seg == NULL))
+    return;
+
+  segment %= buf->spec.segtotal;
 
   data = GST_BUFFER_DATA (buf->data);
-  data += (segment % buf->spec.segtotal) * buf->spec.segsize;
+  data += segment * buf->spec.segsize;
 
   GST_LOG ("clear segment %d @%p", segment, data);
 
