@@ -108,6 +108,7 @@ gst_ffmpegcsp_caps_remove_format_info (GstCaps * caps)
     gst_structure_remove_field (structure, "green_mask");
     gst_structure_remove_field (structure, "blue_mask");
     gst_structure_remove_field (structure, "alpha_mask");
+    gst_structure_remove_field (structure, "palette_data");
   }
 
   gst_caps_do_simplify (caps);
@@ -147,11 +148,13 @@ gst_ffmpegcsp_transform_caps (GstBaseTransform * btrans,
 
   template = gst_ffmpegcsp_codectype_to_caps (CODEC_TYPE_VIDEO, NULL);
   result = gst_caps_intersect (caps, template);
+  gst_caps_unref (template);
 
   gst_caps_append (result, gst_ffmpegcsp_caps_remove_format_info (caps));
 
   GST_DEBUG_OBJECT (btrans, "transformed %" GST_PTR_FORMAT " into %"
       GST_PTR_FORMAT, caps, result);
+
   return result;
 }
 
@@ -231,6 +234,7 @@ gst_ffmpegcsp_set_caps (GstBaseTransform * btrans, GstCaps * incaps,
   if (space->palette)
     av_free (space->palette);
   space->palette = ctx->palctrl;
+  ctx->palctrl = NULL;
 
   /* get to format */
   ctx->pix_fmt = PIX_FMT_NB;
@@ -248,28 +252,28 @@ gst_ffmpegcsp_set_caps (GstBaseTransform * btrans, GstCaps * incaps,
   /* ERRORS */
 no_width_height:
   {
-    GST_DEBUG ("did not specify width or height");
+    GST_DEBUG_OBJECT (space, "did not specify width or height");
     space->from_pixfmt = PIX_FMT_NB;
     space->to_pixfmt = PIX_FMT_NB;
     return FALSE;
   }
 no_framerate:
   {
-    GST_DEBUG ("did not specify framerate");
+    GST_DEBUG_OBJECT (space, "did not specify framerate");
     space->from_pixfmt = PIX_FMT_NB;
     space->to_pixfmt = PIX_FMT_NB;
     return FALSE;
   }
 format_mismatch:
   {
-    GST_DEBUG ("input and output formats do not match");
+    GST_DEBUG_OBJECT (space, "input and output formats do not match");
     space->from_pixfmt = PIX_FMT_NB;
     space->to_pixfmt = PIX_FMT_NB;
     return FALSE;
   }
 invalid_in_caps:
   {
-    GST_DEBUG ("could not configure context for input format");
+    GST_DEBUG_OBJECT (space, "could not configure context for input format");
     av_free (ctx);
     space->from_pixfmt = PIX_FMT_NB;
     space->to_pixfmt = PIX_FMT_NB;
@@ -277,7 +281,7 @@ invalid_in_caps:
   }
 invalid_out_caps:
   {
-    GST_DEBUG ("could not configure context for output format");
+    GST_DEBUG_OBJECT (space, "could not configure context for output format");
     av_free (ctx);
     space->from_pixfmt = PIX_FMT_NB;
     space->to_pixfmt = PIX_FMT_NB;
@@ -304,7 +308,7 @@ gst_ffmpegcsp_get_type (void)
     };
 
     ffmpegcsp_type = g_type_register_static (GST_TYPE_BASE_TRANSFORM,
-        "GstFFMpegColorspace", &ffmpegcsp_info, 0);
+        "GstFFMpegCsp", &ffmpegcsp_info, 0);
   }
 
   return ffmpegcsp_type;
@@ -320,6 +324,17 @@ gst_ffmpegcsp_base_init (GstFFMpegCspClass * klass)
   gst_element_class_set_details (element_class, &ffmpegcsp_details);
 }
 
+static void
+gst_ffmpegcsp_finalize (GObject * obj)
+{
+  GstFFMpegCsp *space = GST_FFMPEGCSP (obj);
+
+  if (space->palette)
+    av_free (space->palette);
+
+  G_OBJECT_CLASS (parent_class)->finalize (obj);
+}
+
 static void
 gst_ffmpegcsp_class_init (GstFFMpegCspClass * klass)
 {
@@ -331,7 +346,9 @@ gst_ffmpegcsp_class_init (GstFFMpegCspClass * klass)
   gstelement_class = (GstElementClass *) klass;
   gstbasetransform_class = (GstBaseTransformClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_BASE_TRANSFORM);
+  parent_class = g_type_class_peek_parent (klass);
+
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_ffmpegcsp_finalize);
 
   gstbasetransform_class->transform_caps =
       GST_DEBUG_FUNCPTR (gst_ffmpegcsp_transform_caps);
@@ -383,9 +400,16 @@ gst_ffmpegcsp_get_unit_size (GstBaseTransform * btrans, GstCaps * caps,
 
   *size = avpicture_get_size (ctx->pix_fmt, width, height);
 
-  if (space->palette)
-    *size -= 4 * 256;
+  /* ffmpeg frames have the palette after the frame data, whereas
+   * GStreamer currently puts it into the caps as 'palette_data' field,
+   * so for paletted data the frame size avpicture_get_size() returns is
+   * 1024 bytes larger than what GStreamer expects. */
+  if (gst_structure_has_field (structure, "palette_data")) {
+    *size -= 4 * 256;           /* = AVPALETTE_SIZE */
+  }
 
+  if (ctx->palctrl)
+    av_free (ctx->palctrl);
   av_free (ctx);
 
   return TRUE;
@@ -406,6 +430,7 @@ gst_ffmpegcsp_transform (GstBaseTransform * btrans, GstBuffer * inbuf,
     GstBuffer * outbuf)
 {
   GstFFMpegCsp *space;
+  gint result;
 
   space = GST_FFMPEGCSP (btrans);
 
@@ -426,8 +451,10 @@ gst_ffmpegcsp_transform (GstBaseTransform * btrans, GstBuffer * inbuf,
       GST_BUFFER_DATA (outbuf), space->to_pixfmt, space->width, space->height);
 
   /* and convert */
-  img_convert (&space->to_frame, space->to_pixfmt,
+  result = img_convert (&space->to_frame, space->to_pixfmt,
       &space->from_frame, space->from_pixfmt, space->width, space->height);
+  if (result == -1)
+    goto not_supported;
 
   /* copy timestamps */
   gst_buffer_stamp (outbuf, inbuf);
@@ -442,6 +469,12 @@ unknown_format:
         ("attempting to convert colorspaces between unknown formats"));
     return GST_FLOW_NOT_NEGOTIATED;
   }
+not_supported:
+  {
+    GST_ELEMENT_ERROR (space, CORE, NOT_IMPLEMENTED, (NULL),
+        ("cannot convert between formats"));
+    return GST_FLOW_NOT_SUPPORTED;
+  }
 }
 
 gboolean
