@@ -62,6 +62,8 @@ struct _GstDecodeBin
 
   GList *queues;                /* list of demuxer-decoder queues */
 
+  GList *probes;                /* list of PadProbeData */
+
   GList *factories;             /* factories we can use for selecting elements */
   gint numpads;
   gint numwaiting;
@@ -95,6 +97,14 @@ enum
   LAST_SIGNAL
 };
 
+
+typedef struct
+{
+  GstPad *pad;
+  gulong sigid;
+  gboolean done;
+} PadProbeData;
+
 /* this structure is created for all dynamic pads that could get created
  * at runtime */
 typedef struct
@@ -336,6 +346,7 @@ gst_decode_bin_init (GstDecodeBin * decode_bin)
 
   decode_bin->dynamics = NULL;
   decode_bin->queues = NULL;
+  decode_bin->probes = NULL;
 }
 
 static void dynamic_free (GstDynamic * dyn);
@@ -460,7 +471,24 @@ mimetype_is_raw (const gchar * mimetype)
 {
   return g_str_has_prefix (mimetype, "video/x-raw") ||
       g_str_has_prefix (mimetype, "audio/x-raw") ||
-      g_str_has_prefix (mimetype, "text/plain");
+      g_str_has_prefix (mimetype, "text/plain") ||
+      g_str_has_prefix (mimetype, "text/x-pango-markup");
+}
+
+static void
+free_pad_probes (GstDecodeBin * decode_bin)
+{
+  GList *tmp;
+
+  /* Remove pad probes */
+  for (tmp = decode_bin->probes; tmp; tmp = g_list_next (tmp)) {
+    PadProbeData *data = (PadProbeData *) tmp->data;
+
+    gst_pad_remove_data_probe (data->pad, data->sigid);
+    g_free (data);
+  }
+  g_list_free (decode_bin->probes);
+  decode_bin->probes = NULL;
 }
 
 static void
@@ -477,22 +505,41 @@ remove_fakesink (GstDecodeBin * decode_bin)
     gst_object_unref (decode_bin->fakesink);
     decode_bin->fakesink = NULL;
 
+    free_pad_probes (decode_bin);
+
     gst_element_post_message (GST_ELEMENT_CAST (decode_bin),
         gst_message_new_state_dirty (GST_OBJECT_CAST (decode_bin)));
   }
 }
 
-static void
-pad_blocked (GstPad * pad, gboolean blocked, GstDecodeBin * decode_bin)
+static gboolean
+pad_probe (GstPad * pad, GstMiniObject * data, GstDecodeBin * decode_bin)
 {
-  if (blocked) {
-    decode_bin->numwaiting--;
-    if (decode_bin->numwaiting == 0) {
-      remove_fakesink (decode_bin);
-    }
-    gst_pad_set_blocked_async (pad, FALSE, (GstPadBlockCallback) pad_blocked,
-        NULL);
+  GList *tmp;
+  gboolean alldone = TRUE;
+
+  for (tmp = decode_bin->probes; tmp; tmp = g_list_next (tmp)) {
+    PadProbeData *pdata = (PadProbeData *) tmp->data;
+
+    if (pdata->pad == pad) {
+      if (GST_IS_BUFFER (data)) {
+        if (!pdata->done)
+          decode_bin->numwaiting--;
+        pdata->done = TRUE;
+      } else if (GST_IS_EVENT (data) &&
+          ((GST_EVENT_TYPE (data) == GST_EVENT_EOS) ||
+              (GST_EVENT_TYPE (data) == GST_EVENT_TAG) ||
+              (GST_EVENT_TYPE (data) == GST_EVENT_FLUSH_START))) {
+        if (!pdata->done)
+          decode_bin->numwaiting--;
+        pdata->done = TRUE;
+      }
+    } else if (!(pdata->done))
+      alldone = FALSE;
   }
+  if (alldone)
+    remove_fakesink (decode_bin);
+  return TRUE;
 }
 
 /* given a pad and a caps from an element, find the list of elements
@@ -544,6 +591,7 @@ close_pad_link (GstElement * element, GstPad * pad, GstCaps * caps,
   if (mimetype_is_raw (mimetype)) {
     gchar *padname;
     GstPad *ghost;
+    PadProbeData *data;
 
     /* make a unique name for this new pad */
     padname = g_strdup_printf ("src%d", decode_bin->numpads);
@@ -553,10 +601,15 @@ close_pad_link (GstElement * element, GstPad * pad, GstCaps * caps,
     ghost = gst_ghost_pad_new (padname, pad);
     gst_element_add_pad (GST_ELEMENT (decode_bin), ghost);
 
-    if (gst_pad_set_blocked_async (pad, TRUE, (GstPadBlockCallback) pad_blocked,
-            decode_bin)) {
-      decode_bin->numwaiting++;
-    }
+    data = g_new0 (PadProbeData, 1);
+    data->pad = pad;
+    data->done = FALSE;
+
+    data->sigid = gst_pad_add_data_probe (pad, G_CALLBACK (pad_probe),
+        decode_bin);
+    decode_bin->numwaiting++;
+
+    decode_bin->probes = g_list_append (decode_bin->probes, data);
 
     GST_LOG_OBJECT (element, "closed pad %s", padname);
 
@@ -677,6 +730,8 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstElement * srcelement, GstPad * pad,
   isdemux = is_demuxer_element (srcelement);
 
   if (isdemux && factories != NULL) {
+    GstPadLinkReturn dqlink;
+
     /* Insert a queue between demuxer and decoder */
     GST_DEBUG_OBJECT (decode_bin,
         "Element %s is a demuxer, inserting a queue",
@@ -685,15 +740,16 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstElement * srcelement, GstPad * pad,
     decode_bin->queue_type = G_OBJECT_TYPE (queue);
 
     g_object_set (G_OBJECT (queue), "max-size-buffers", 0, NULL);
-    g_object_set (G_OBJECT (queue), "max-size-time", 0LL, NULL);
+    g_object_set (G_OBJECT (queue), "max-size-time", G_GINT64_CONSTANT (0),
+        NULL);
     g_object_set (G_OBJECT (queue), "max-size-bytes", 8192, NULL);
     gst_bin_add (GST_BIN (decode_bin), queue);
     gst_element_set_state (queue, GST_STATE_READY);
     queuesinkpad = gst_element_get_pad (queue, "sink");
     usedsrcpad = queuesrcpad = gst_element_get_pad (queue, "src");
 
-    g_return_val_if_fail (gst_pad_link (pad, queuesinkpad) == GST_PAD_LINK_OK,
-        NULL);
+    dqlink = gst_pad_link (pad, queuesinkpad);
+    g_return_val_if_fail (dqlink == GST_PAD_LINK_OK, NULL);
   }
 
   /* loop over the factories */
@@ -785,7 +841,7 @@ try_to_link_1 (GstDecodeBin * decode_bin, GstElement * srcelement, GstPad * pad,
 done:
   if (queue != NULL) {
     /* We didn't successfully connect to the queue */
-    gst_pad_unlink (pad, queuesrcpad);
+    gst_pad_unlink (pad, queuesinkpad);
     gst_element_set_state (queue, GST_STATE_NULL);
     gst_object_unref (queuesrcpad);
     gst_object_unref (queuesinkpad);
@@ -797,45 +853,55 @@ done:
 static GstPad *
 get_our_ghost_pad (GstDecodeBin * decode_bin, GstPad * pad)
 {
-#if 0
-  GList *ghostpads;
+  GstIterator *pad_it = NULL;
+  GstPad *db_pad = NULL;
+  gboolean done = FALSE;
 
   if (pad == NULL || !GST_PAD_IS_SRC (pad)) {
     GST_DEBUG_OBJECT (decode_bin, "pad NULL or not SRC pad");
     return NULL;
   }
 
-  if (GST_IS_GHOST_PAD (pad)) {
-    GstElement *parent = gst_pad_get_parent (pad);
+  pad_it = gst_element_iterate_pads (GST_ELEMENT (decode_bin));
+  while (!done) {
+    switch (gst_iterator_next (pad_it, (gpointer) & db_pad)) {
+      case GST_ITERATOR_OK:
+        GST_DEBUG_OBJECT (decode_bin, "looking at pad %s:%s",
+            GST_DEBUG_PAD_NAME (db_pad));
+        if (GST_IS_GHOST_PAD (db_pad) && GST_PAD_IS_SRC (db_pad)) {
+          GstPad *target_pad = NULL;
 
-    GST_DEBUG_OBJECT (decode_bin, "pad parent %s", GST_ELEMENT_NAME (parent));
+          target_pad = gst_ghost_pad_get_target (GST_GHOST_PAD (db_pad));
 
-    if (parent == GST_ELEMENT (decode_bin)) {
-      GST_DEBUG_OBJECT (decode_bin, "pad is our ghostpad");
-      gst_object_unref (parent);
-      return pad;
-    } else {
-      GST_DEBUG_OBJECT (decode_bin, "pad is ghostpad but not ours");
-      gst_object_unref (parent);
-      return NULL;
+          if (target_pad == pad) {      /* Found our ghost pad */
+            GST_DEBUG_OBJECT (decode_bin, "found ghostpad %s:%s for pad %s:%s",
+                GST_DEBUG_PAD_NAME (db_pad), GST_DEBUG_PAD_NAME (pad));
+            done = TRUE;
+            break;
+          } else {              /* Not the right one */
+            gst_object_unref (db_pad);
+            db_pad = NULL;
+          }
+        } else {
+          gst_object_unref (db_pad);
+          db_pad = NULL;
+        }
+
+        break;
+      case GST_ITERATOR_RESYNC:
+        gst_iterator_resync (pad_it);
+        break;
+      case GST_ITERATOR_ERROR:
+        done = TRUE;
+        break;
+      case GST_ITERATOR_DONE:
+        done = TRUE;
+        break;
     }
   }
+  gst_iterator_free (pad_it);
 
-  GST_DEBUG_OBJECT (decode_bin, "looping over ghostpads");
-  ghostpads = GST_REAL_PAD (pad)->ghostpads;
-  while (ghostpads) {
-    GstPad *ghostpad;
-
-    ghostpad = get_our_ghost_pad (decode_bin, GST_PAD (ghostpads->data));
-    if (ghostpad)
-      return ghostpad;
-
-    ghostpads = g_list_next (ghostpads);
-  }
-  GST_DEBUG_OBJECT (decode_bin, "done looping over ghostpads, nothing found");
-#endif
-
-  return NULL;
+  return db_pad;
 }
 
 /* remove all downstream elements starting from the given pad.
@@ -881,6 +947,7 @@ remove_element_chain (GstDecodeBin * decode_bin, GstPad * pad)
           gst_decode_bin_signals[SIGNAL_REMOVED_DECODED_PAD], 0, ghostpad);
 
       gst_element_remove_pad (GST_ELEMENT (decode_bin), ghostpad);
+      gst_object_unref (ghostpad);
       continue;
     } else {
       GST_DEBUG_OBJECT (decode_bin, "not one of our ghostpads");
@@ -894,16 +961,21 @@ remove_element_chain (GstDecodeBin * decode_bin, GstPad * pad)
         GST_DEBUG_PAD_NAME (pad), GST_DEBUG_PAD_NAME (peer));
 
     {
-      GstElement *parent = gst_pad_get_parent_element (peer);
+      GstObject *parent = gst_pad_get_parent (peer);
 
       if (parent) {
-        if (parent != GST_ELEMENT (decode_bin)) {
-          GST_DEBUG_OBJECT (decode_bin, "dead end pad %s:%s",
-              GST_DEBUG_PAD_NAME (peer));
-        } else {
-          GST_DEBUG_OBJECT (decode_bin, "recursing element %s on pad %s:%s",
-              GST_ELEMENT_NAME (elem), GST_DEBUG_PAD_NAME (pad));
-          remove_element_chain (decode_bin, peer);
+        GstElement *grandparent = GST_ELEMENT (gst_object_get_parent (parent));
+
+        if (grandparent) {
+          if (grandparent != GST_ELEMENT (decode_bin)) {
+            GST_DEBUG_OBJECT (decode_bin, "dead end pad %s:%s parent %s",
+                GST_DEBUG_PAD_NAME (peer), GST_OBJECT_NAME (grandparent));
+          } else {
+            GST_DEBUG_OBJECT (decode_bin, "recursing element %s on pad %s:%s",
+                GST_ELEMENT_NAME (elem), GST_DEBUG_PAD_NAME (pad));
+            remove_element_chain (decode_bin, peer);
+          }
+          gst_object_unref (grandparent);
         }
         gst_object_unref (parent);
       }
@@ -1240,6 +1312,81 @@ shutting_down:
   return;
 }
 
+static void
+cleanup_decodebin (GstDecodeBin * decode_bin)
+{
+  GstIterator *elem_it = NULL, *gpad_it = NULL;
+  GstPad *typefind_pad = NULL;
+  gboolean done = FALSE;
+
+  g_return_if_fail (GST_IS_DECODE_BIN (decode_bin));
+
+  GST_DEBUG_OBJECT (decode_bin, "cleaning up decodebin");
+
+  typefind_pad = gst_element_get_pad (decode_bin->typefind, "src");
+  if (GST_IS_PAD (typefind_pad)) {
+    g_signal_handlers_block_by_func (typefind_pad, unlinked, decode_bin);
+  }
+
+  elem_it = gst_bin_iterate_elements (GST_BIN (decode_bin));
+  while (!done) {
+    GstElement *element = NULL;
+
+    switch (gst_iterator_next (elem_it, (gpointer) & element)) {
+      case GST_ITERATOR_OK:
+        if (element != decode_bin->typefind && element != decode_bin->fakesink) {
+          GST_DEBUG_OBJECT (element, "removing autoplugged element");
+          g_signal_handlers_disconnect_by_func (element, unlinked, decode_bin);
+          gst_bin_remove (GST_BIN (decode_bin), element);
+        }
+        gst_object_unref (element);
+        break;
+      case GST_ITERATOR_RESYNC:
+        gst_iterator_resync (elem_it);
+        break;
+      case GST_ITERATOR_ERROR:
+        done = TRUE;
+        break;
+      case GST_ITERATOR_DONE:
+        done = TRUE;
+        break;
+    }
+  }
+  gst_iterator_free (elem_it);
+
+  gpad_it = gst_element_iterate_pads (GST_ELEMENT (decode_bin));
+  while (!done) {
+    GstPad *pad = NULL;
+
+    switch (gst_iterator_next (gpad_it, (gpointer) & pad)) {
+      case GST_ITERATOR_OK:
+        GST_DEBUG_OBJECT (pad, "inspecting pad %s:%s",
+            GST_DEBUG_PAD_NAME (pad));
+        if (GST_IS_GHOST_PAD (pad) && GST_PAD_IS_SRC (pad)) {
+          GST_DEBUG_OBJECT (pad, "removing ghost pad");
+          gst_element_remove_pad (GST_ELEMENT (decode_bin), pad);
+        }
+        gst_object_unref (pad);
+        break;
+      case GST_ITERATOR_RESYNC:
+        gst_iterator_resync (gpad_it);
+        break;
+      case GST_ITERATOR_ERROR:
+        done = TRUE;
+        break;
+      case GST_ITERATOR_DONE:
+        done = TRUE;
+        break;
+    }
+  }
+  gst_iterator_free (gpad_it);
+
+  if (GST_IS_PAD (typefind_pad)) {
+    g_signal_handlers_unblock_by_func (typefind_pad, unlinked, decode_bin);
+    gst_object_unref (typefind_pad);
+  }
+}
+
 static GstStateChangeReturn
 gst_decode_bin_change_state (GstElement * element, GstStateChange transition)
 {
@@ -1279,6 +1426,8 @@ gst_decode_bin_change_state (GstElement * element, GstStateChange transition)
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       free_dynamics (decode_bin);
+      free_pad_probes (decode_bin);
+      cleanup_decodebin (decode_bin);
       break;
     default:
       break;
