@@ -28,6 +28,7 @@
 #include <regex.h>
 
 #include "gstsubparse.h"
+#include "gstssaparse.h"
 
 GST_DEBUG_CATEGORY_STATIC (sub_parse_debug);
 #define GST_CAT_DEFAULT sub_parse_debug
@@ -382,6 +383,44 @@ parse_mdvdsub (ParserState * state, const gchar * line)
   return ret;
 }
 
+/* we want to escape text in general, but retain basic markup like
+ * <i></i>, <u></u>, and <b></b>. The easiest and safest way is to
+ * just unescape a white list of allowed markups again after
+ * escaping everything (the text between these simple markers isn't
+ * necessarily escaped, so it seems best to do it like this) */
+static void
+subrip_unescape_formatting (gchar * txt)
+{
+  gchar *pos;
+
+  for (pos = txt; pos != NULL && *pos != '\0'; ++pos) {
+    if (g_ascii_strncasecmp (pos, "&lt;u&gt;", 9) == 0 ||
+        g_ascii_strncasecmp (pos, "&lt;i&gt;", 9) == 0 ||
+        g_ascii_strncasecmp (pos, "&lt;b&gt;", 9) == 0) {
+      pos[0] = '<';
+      pos[1] = g_ascii_tolower (pos[4]);
+      pos[2] = '>';
+      /* move NUL terminator as well */
+      g_memmove (pos + 3, pos + 9, strlen (pos + 9) + 1);
+      pos += 2;
+    }
+  }
+
+  for (pos = txt; pos != NULL && *pos != '\0'; ++pos) {
+    if (g_ascii_strncasecmp (pos, "&lt;/u&gt;", 10) == 0 ||
+        g_ascii_strncasecmp (pos, "&lt;/i&gt;", 10) == 0 ||
+        g_ascii_strncasecmp (pos, "&lt;/b&gt;", 10) == 0) {
+      pos[0] = '<';
+      pos[1] = '/';
+      pos[2] = g_ascii_tolower (pos[5]);
+      pos[3] = '>';
+      /* move NUL terminator as well */
+      g_memmove (pos + 4, pos + 10, strlen (pos + 10) + 1);
+      pos += 3;
+    }
+  }
+}
+
 static gchar *
 parse_subrip (ParserState * state, const gchar * line)
 {
@@ -439,6 +478,7 @@ parse_subrip (ParserState * state, const gchar * line)
         ret = g_markup_escape_text (state->buf->str, state->buf->len);
         g_string_truncate (state->buf, 0);
         state->state = 0;
+        subrip_unescape_formatting (ret);
         return ret;
       }
       return NULL;
@@ -595,7 +635,7 @@ gst_sub_parse_format_autodetect (GstSubParse * self)
       return gst_caps_new_simple ("text/x-pango-markup", NULL);
     case GST_SUB_PARSE_FORMAT_SUBRIP:
       self->parse_line = parse_subrip;
-      return gst_caps_new_simple ("text/plain", NULL);
+      return gst_caps_new_simple ("text/x-pango-markup", NULL);
     case GST_SUB_PARSE_FORMAT_MPSUB:
       self->parse_line = parse_mpsub;
       return gst_caps_new_simple ("text/plain", NULL);
@@ -827,10 +867,16 @@ gst_subparse_type_find (GstTypeFind * tf, gpointer private)
 {
   const guint8 *data;
   GstSubParseFormat format;
+  gchar *str;
 
   if (!(data = gst_type_find_peek (tf, 0, 36)))
     return;
-  format = gst_sub_parse_data_format_autodetect ((gchar *) data);
+
+  /* make sure string passed to _autodetect() is NUL-terminated */
+  str = g_strndup ((gchar *) data, 35);
+  format = gst_sub_parse_data_format_autodetect (str);
+  g_free (str);
+
   switch (format) {
     case GST_SUB_PARSE_FORMAT_MDVDSUB:
       GST_DEBUG ("MicroDVD format detected");
@@ -839,7 +885,7 @@ gst_subparse_type_find (GstTypeFind * tf, gpointer private)
       GST_DEBUG ("SubRip format detected");
       break;
     case GST_SUB_PARSE_FORMAT_MPSUB:
-      GST_DEBUG ("SubRip format detected");
+      GST_DEBUG ("MPSub format detected");
       break;
     case GST_SUB_PARSE_FORMAT_UNKNOWN:
       GST_DEBUG ("no subtitle format detected");
@@ -861,12 +907,18 @@ plugin_init (GstPlugin * plugin)
           gst_subparse_type_find, sub_exts, SUB_CAPS, NULL, NULL))
     return FALSE;
 
-  return gst_element_register (plugin, "subparse",
-      GST_RANK_PRIMARY, GST_TYPE_SUBPARSE);
+  if (!gst_element_register (plugin, "subparse",
+          GST_RANK_PRIMARY, GST_TYPE_SUBPARSE) ||
+      !gst_element_register (plugin, "ssaparse",
+          GST_RANK_PRIMARY, GST_TYPE_SSA_PARSE)) {
+    return FALSE;
+  }
+
+  return TRUE;
 }
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
     "subparse",
-    "Subtitle (.sub) file parsing",
+    "Subtitle parsing",
     plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN)
